'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.default = withNavigation;

var _constants = require('../constants');

var _constants2 = _interopRequireDefault(_constants);

var _transition = require('../transition');

var _transition2 = _interopRequireDefault(_transition);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var noop = function noop() {};

function withNavigation(router) {
    var cancelCurrentTransition = void 0;

    router.navigate = navigate;
    router.navigateToDefault = navigateToDefault;
    router.transitionToState = transitionToState;
    router.cancel = cancel;

    function cancel() {
        if (cancelCurrentTransition) {
            cancelCurrentTransition('navigate');
            cancelCurrentTransition = null;
        }

        return router;
    }

    function navigate() {
        var name = arguments.length <= 0 ? undefined : arguments[0];
        var lastArg = arguments.length <= arguments.length - 1 + 0 ? undefined : arguments[arguments.length - 1 + 0];
        var done = typeof lastArg === 'function' ? lastArg : noop;
        var params = _typeof(arguments.length <= 1 ? undefined : arguments[1]) === 'object' ? arguments.length <= 1 ? undefined : arguments[1] : {};
        var opts = _typeof(arguments.length <= 2 ? undefined : arguments[2]) === 'object' ? arguments.length <= 2 ? undefined : arguments[2] : {};

        if (!router.isStarted()) {
            done({ code: _constants.errorCodes.ROUTER_NOT_STARTED });
            return;
        }

        var toState = router.buildState(name, params);

        if (!toState) {
            var err = { code: _constants.errorCodes.ROUTE_NOT_FOUND };
            done(err);
            router.invokeEventListeners(_constants2.default.TRANSITION_ERROR, null, router.getState(), err);
            return;
        }

        toState.path = router.buildPath(name, params);
        var sameStates = router.getState() ? router.areStatesEqual(router.getState(), toState, false) : false;

        // Do not proceed further if states are the same and no reload
        // (no desactivation and no callbacks)
        if (sameStates && !opts.reload) {
            var _err = { code: _constants.errorCodes.SAME_STATES };
            done(_err);
            router.invokeEventListeners(_constants2.default.TRANSITION_ERROR, toState, router.getState(), _err);
            return;
        }

        var fromState = sameStates ? null : router.getState();

        // Transitio
        return transitionToState(toState, fromState, opts, function (err, state) {
            if (err) {
                if (err.redirect) {
                    var _err$redirect = err.redirect;
                    var _name = _err$redirect.name;
                    var _params = _err$redirect.params;


                    navigate(_name, _params, _extends({}, opts, { reload: true }), done);
                } else {
                    done(err);
                }
            } else {
                router.invokeEventListeners(_constants2.default.TRANSITION_SUCCESS, state, fromState, opts);
                done(null, state);
            }
        });
    }

    function navigateToDefault() {
        var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
        var done = arguments.length <= 1 || arguments[1] === undefined ? noop : arguments[1];

        var options = router.getOptions();

        return navigate(options.defaultRoute, options.defaultParams, opts, done);
    }

    function transitionToState(toState, fromState) {
        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
        var done = arguments.length <= 3 || arguments[3] === undefined ? noop : arguments[3];

        cancel();
        router.invokeEventListeners(_constants2.default.TRANSITION_START, toState, fromState);

        cancelCurrentTransition = (0, _transition2.default)(router, toState, fromState, options, function (err, state) {
            cancelCurrentTransition = null;
            state = state || toState;

            if (err) {
                if (err.code === _constants.errorCodes.TRANSITION_CANCELLED) {
                    router.invokeEventListeners(_constants2.default.TRANSITION_CANCELLED, toState, fromState);
                } else {
                    router.invokeEventListeners(_constants2.default.TRANSITION_ERROR, toState, fromState, err);
                }
                done(err);
            } else {
                router.setState(state);
                done(null, state);
            }
        });

        return cancelCurrentTransition;
    }
}