'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _routeNode = require('route-node');

var _routeNode2 = _interopRequireDefault(_routeNode);

var _utils = require('./core/utils');

var _utils2 = _interopRequireDefault(_utils);

var _routerLifecycle = require('./core/router-lifecycle');

var _routerLifecycle2 = _interopRequireDefault(_routerLifecycle);

var _navigation = require('./core/navigation');

var _navigation2 = _interopRequireDefault(_navigation);

var _middleware = require('./core/middleware');

var _middleware2 = _interopRequireDefault(_middleware);

var _plugins = require('./core/plugins');

var _plugins2 = _interopRequireDefault(_plugins);

var _routeLifecycle = require('./core/route-lifecycle');

var _routeLifecycle2 = _interopRequireDefault(_routeLifecycle);

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultOptions = {
    trailingSlash: 0,
    autoCleanUp: true,
    strictQueryParams: true,
    allowNotFound: false
};

function createRouter(routes, opts) {
    var routerState = null;
    var callbacks = {};
    var dependencies = {};
    var options = _extends({}, defaultOptions, opts);

    var router = {
        rootNode: rootNode,
        getOptions: getOptions,
        setOption: setOption,
        getState: getState,
        setState: setState,
        makeState: makeState,
        makeNotFoundState: makeNotFoundState,
        setDependency: setDependency,
        setDependencies: setDependencies,
        getDependencies: getDependencies,
        add: add,
        addNode: addNode,
        executeFactory: executeFactory,
        addEventListener: addEventListener,
        removeEventListener: removeEventListener,
        invokeEventListeners: invokeEventListeners
    };

    function invokeEventListeners(eventName) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
        }

        (callbacks[eventName] || []).forEach(function (cb) {
            return cb.apply(undefined, args);
        });
    }

    function removeEventListener(eventName, cb) {
        callbacks[eventName] = callbacks[eventName].filter(function (_cb) {
            return _cb !== cb;
        });
    }

    function addEventListener(eventName, cb) {
        callbacks[eventName] = (callbacks[eventName] || []).concat(cb);

        return function () {
            return removeEventListener(eventName, cb);
        };
    }

    (0, _utils2.default)(router);
    (0, _plugins2.default)(router);
    (0, _middleware2.default)(router);
    (0, _routeLifecycle2.default)(router);
    (0, _routerLifecycle2.default)(router);
    (0, _navigation2.default)(router);

    var rootNode = routes instanceof _routeNode2.default ? routes : new _routeNode2.default('', '', routes, addCanActivate);

    router.rootNode = rootNode;

    return router;

    function addCanActivate(route) {
        if (route.canActivate) router.canActivate(route.name, route.canActivate);
    }

    function makeState(name, params, path, metaParams, source) {
        var state = {};
        var setProp = function setProp(key, value) {
            return Object.defineProperty(state, key, { value: value, enumerable: true });
        };
        setProp('name', name);
        setProp('params', params);
        setProp('path', path);
        if (metaParams || source) {
            var meta = { params: metaParams };

            if (source) meta.source = source;

            setProp('meta', meta);
        }
        return state;
    }

    function makeNotFoundState(path) {
        return makeState(_constants2.default.UNKNOWN_ROUTE, { path: path }, path, {});
    }

    function getState() {
        return routerState;
    }

    function setState(state) {
        routerState = state;
    }

    function getOptions() {
        return options;
    }

    function setOption(option, value) {
        options[option] = value;
        return router;
    }

    function setDependency(dependencyName, dependency) {
        dependencies[dependencyName] = dependency;
        return router;
    }

    function setDependencies(deps) {
        Object.keys(deps).forEach(function (depName) {
            dependencies[depName] = deps[depName];
        });
    }

    function getDependencies() {
        return dependencies;
    }

    function getInjectables() {
        return [router, dependencies];
    }

    function executeFactory(factoryFunction) {
        return factoryFunction.apply(undefined, _toConsumableArray(getInjectables()));
    }

    function add(routes) {
        rootNode.add(routes, addCanActivate);
        return router;
    }

    function addNode(name, path, canActivateHandler) {
        rootNode.addNode(name, path);
        if (canActivateHandler) router.canActivate(name, canActivateHandler);
        return router;
    }
}

exports.default = createRouter;