import RouteNode from 'route-node';
import withUtils from './core/utils';
import withRouterLifecycle from './core/router-lifecycle';
import withNavigation from './core/navigation';
import withMiddleware from './core/middleware';
import withPlugins from './core/plugins';
import withRouteLifecycle from './core/route-lifecycle';
import constants from './constants';

var defaultOptions = {
    trailingSlash: 0,
    autoCleanUp: true,
    strictQueryParams: true,
    allowNotFound: false
};

function createRouter(routes, opts) {
    var routerState = null;
    var callbacks = {};
    var dependencies = {};
    var options = babelHelpers.extends({}, defaultOptions, opts);

    var router = {
        rootNode: rootNode,
        getOptions: getOptions,
        setOption: setOption,
        getState: getState,
        setState: setState,
        makeState: makeState,
        makeNotFoundState: makeNotFoundState,
        setDependency: setDependency,
        setDependencies: setDependencies,
        getDependencies: getDependencies,
        add: add,
        addNode: addNode,
        executeFactory: executeFactory,
        addEventListener: addEventListener,
        removeEventListener: removeEventListener,
        invokeEventListeners: invokeEventListeners
    };

    function invokeEventListeners(eventName) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
        }

        (callbacks[eventName] || []).forEach(function (cb) {
            return cb.apply(undefined, args);
        });
    }

    function removeEventListener(eventName, cb) {
        callbacks[eventName] = callbacks[eventName].filter(function (_cb) {
            return _cb !== cb;
        });
    }

    function addEventListener(eventName, cb) {
        callbacks[eventName] = (callbacks[eventName] || []).concat(cb);

        return function () {
            return removeEventListener(eventName, cb);
        };
    }

    withUtils(router);
    withPlugins(router);
    withMiddleware(router);
    withRouteLifecycle(router);
    withRouterLifecycle(router);
    withNavigation(router);

    var rootNode = routes instanceof RouteNode ? routes : new RouteNode('', '', routes, addCanActivate);

    router.rootNode = rootNode;

    return router;

    function addCanActivate(route) {
        if (route.canActivate) router.canActivate(route.name, route.canActivate);
    }

    function makeState(name, params, path, metaParams, source) {
        var state = {};
        var setProp = function setProp(key, value) {
            return Object.defineProperty(state, key, { value: value, enumerable: true });
        };
        setProp('name', name);
        setProp('params', params);
        setProp('path', path);
        if (metaParams || source) {
            var meta = { params: metaParams };

            if (source) meta.source = source;

            setProp('meta', meta);
        }
        return state;
    }

    function makeNotFoundState(path) {
        return makeState(constants.UNKNOWN_ROUTE, { path: path }, path, {});
    }

    function getState() {
        return routerState;
    }

    function setState(state) {
        routerState = state;
    }

    function getOptions() {
        return options;
    }

    function setOption(option, value) {
        options[option] = value;
        return router;
    }

    function setDependency(dependencyName, dependency) {
        dependencies[dependencyName] = dependency;
        return router;
    }

    function setDependencies(deps) {
        Object.keys(deps).forEach(function (depName) {
            dependencies[depName] = deps[depName];
        });
    }

    function getDependencies() {
        return dependencies;
    }

    function getInjectables() {
        return [router, dependencies];
    }

    function executeFactory(factoryFunction) {
        return factoryFunction.apply(undefined, babelHelpers.toConsumableArray(getInjectables()));
    }

    function add(routes) {
        rootNode.add(routes, addCanActivate);
        return router;
    }

    function addNode(name, path, canActivateHandler) {
        rootNode.addNode(name, path);
        if (canActivateHandler) router.canActivate(name, canActivateHandler);
        return router;
    }
}

export default createRouter;