import transitionPath, { nameToIDs } from 'router5.transition-path';
import resolve from './resolve';
import { errorCodes } from '../constants';

export default transition;

function transition(router, toState, fromState, opts, callback) {
    var cancelled = false;
    var completed = false;
    var options = router.getOptions();

    var _router$getLifecycleF = router.getLifecycleFunctions();

    var _router$getLifecycleF2 = babelHelpers.slicedToArray(_router$getLifecycleF, 2);

    var canDeactivateFunctions = _router$getLifecycleF2[0];
    var canActivateFunctions = _router$getLifecycleF2[1];

    var middlewareFunctions = router.getMiddlewareFunctions();
    var isCancelled = function isCancelled() {
        return cancelled;
    };
    var cancel = function cancel() {
        if (!cancelled && !completed) {
            cancelled = true;
            callback({ code: errorCodes.TRANSITION_CANCELLED }, null);
        }
    };
    var done = function done(err, state) {
        completed = true;

        if (isCancelled()) {
            return;
        }

        if (!err && options.autoCleanUp) {
            (function () {
                var activeSegments = nameToIDs(toState.name);
                Object.keys(canDeactivateFunctions).forEach(function (name) {
                    if (activeSegments.indexOf(name) === -1) router.clearCanDeactivate(name);
                });
            })();
        }

        callback(err, state || toState);
    };
    var makeError = function makeError(base, err) {
        return babelHelpers.extends({}, base, err instanceof Object ? err : { error: err });
    };

    var _transitionPath = transitionPath(toState, fromState);

    var toDeactivate = _transitionPath.toDeactivate;
    var toActivate = _transitionPath.toActivate;

    var asyncBase = { isCancelled: isCancelled, toState: toState, fromState: fromState };

    var canDeactivate = function canDeactivate(toState, fromState, cb) {
        var canDeactivateFunctionMap = toDeactivate.filter(function (name) {
            return canDeactivateFunctions[name];
        }).reduce(function (fnMap, name) {
            return babelHelpers.extends({}, fnMap, babelHelpers.defineProperty({}, name, canDeactivateFunctions[name]));
        }, {});

        resolve(canDeactivateFunctionMap, babelHelpers.extends({}, asyncBase, { errorKey: 'segment' }), function (err) {
            return cb(err ? makeError({ code: errorCodes.CANNOT_DEACTIVATE }, err) : null);
        });
    };

    var canActivate = function canActivate(toState, fromState, cb) {
        var canActivateFunctionMap = toActivate.filter(function (name) {
            return canActivateFunctions[name];
        }).reduce(function (fnMap, name) {
            return babelHelpers.extends({}, fnMap, babelHelpers.defineProperty({}, name, canActivateFunctions[name]));
        }, {});

        resolve(canActivateFunctionMap, babelHelpers.extends({}, asyncBase, { errorKey: 'segment' }), function (err) {
            return cb(err ? makeError({ code: errorCodes.CANNOT_ACTIVATE }, err) : null);
        });
    };

    var middleware = !middlewareFunctions.length ? [] : function (toState, fromState, cb) {
        return resolve(middlewareFunctions, babelHelpers.extends({}, asyncBase), function (err, state) {
            return cb(err ? makeError({ code: errorCodes.TRANSITION_ERR }, err) : null, state || toState);
        });
    };

    var pipeline = (fromState && !opts.forceDeactivate ? [canDeactivate] : []).concat(canActivate).concat(middleware);

    resolve(pipeline, asyncBase, done);

    return cancel;
}